#############################################################################
#
# alias.tcl: This file contains procedures related to the processing of the 
#            MH aliases
#
# AliasWindow {}
#       Produce a window with a list of the aliases, and a set of buttons
#       to edit, add and delete aliases. Also a button to compose a message
#       to the selected alias.
#
# DeleteAlias {}
#       Delete an alias from the list
#
# EditAlias {}
#       Change the address for the alias in Addresses
#
# InsertAlias {}
#       Insert an alias before the selected place. If no selection, append 
#       it to the end of the list
#
# ShowAlias {} 
#       Show the selected alias and the Address in a window
#
# SaveAlias {}
#       Save the aliases in the alias file, backing up the old one
#
# SearchAlias {Alias} {resolve 0}
#       Search variable Aliases for the alias, and return the Address. 
#       resolve is a boolean telling to resolve the address further over
#       other aliases, to the real address. all is a boolean telling
#       the procedure if it should output the path of the search.
#       TODO: add a PREF so we can control substitution of aliases
#
# ShowAliases {}
#       Display Aliases var in the listbox
#
##############################################################################

proc AliasWindow {} {
    global Changed BM ALIAS ADDRESS alicat
    set Changed 0

    set alicat [lindex [lsort [array names ALIAS]] 0]
    catch {destroy .alias}
    toplevel .alias -class Config
    wm title .alias "TkMH Aliases"
    wm iconbitmap .alias @$BM(alias)

    frame .alias.p
    label .alias.p.label -width 20 -textvariable alicat -relief sunken -bd 1
    menubutton .alias.p.pull -bitmap @$BM(pull) -menu .alias.p.pull.m -relief raised
    menu .alias.p.pull.m
    foreach name [lsort [array names ALIAS]] {
        .alias.p.pull.m add command -label $name -command "[list set alicat $name];DisplayAliases"
    }

    scrollbar .alias.s -command {.alias.l yview}
    listbox .alias.l -yscroll {.alias.s set} -height 15 -width 15
    
    frame .alias.b
    button .alias.b.done -text "Done" -command {
        catch {destroy .aa}
        if $Changed { if [Confirm "Save Changes?"] {SaveAlias} }
        destroy .alias
        AdaptAliMenus
    }
    button .alias.b.edit -text "Edit" -command {
        if {[set Alias [GetListboxSel .alias.l 1]] != ""} {
            FlashSel .alias.l 2; InsertAlias $Alias $Alias}}
    button .alias.b.delete -text "Delete Alias" -command {DeleteAlias}
    button .alias.b.deletec -text "Delete Category" -command {
        .alias.p.pull.m delete $alicat; DeleteCategory }
    button .alias.b.add -text "New Alias" -command {InsertAlias}
    button .alias.b.addc -text "New Category" -command {InsertCategory}
    button .alias.b.save -text "Save" -command {SaveAlias; AdaptAliMenus}
    button .alias.b.help -text "Help" -command {Help Configuration/Aliases}
    button .alias.b.reset -text "Reset" -command \
       {BuildAliasList; DisplayAliases; AdaptAliMenus; set Changed 1}

    pack append .alias.p \
        .alias.p.label {left expand fill} \
        .alias.p.pull {right expand fill}
    pack append .alias.b \
      .alias.b.edit {top expand fill} \
      .alias.b.add {top expand fill} \
      .alias.b.addc {top expand fill} \
      .alias.b.delete {top expand fill} \
      .alias.b.deletec {top expand fill} \
      .alias.b.help {top expand fill} \
      .alias.b.reset {top expand fill} \
      .alias.b.save {top expand fill} \
      .alias.b.done {top expand fill}
    pack append .alias \
      .alias.p {top expand fill} \
      .alias.l {left expand fill} \
      .alias.s {left filly} \
      .alias.b {left filly}

    bind .alias.l <Double-1> {.alias.b.edit invoke}

    DisplayAliases
}

proc DeleteAlias {} {
    global Changed ALIAS ADDRESS alicat

    if {[set Sels [.alias.l curselection]] == {}} {return}
    FlashSel .alias.l 2
    set first [lindex $Sels 0]
    set last [lindex $Sels [expr [llength $Sels] - 1]]
    set ALIAS($alicat) [lreplace $ALIAS($alicat) $first $last]
    set ADDRESS($alicat) [lreplace $ADDRESS($alicat) $first $last]
    set Changed 1
    DisplayAliases
}

proc DeleteCategory {} {
    global Changed ALIAS ADDRESS alicat

    unset ALIAS($alicat) 
    unset ADDRESS($alicat)
    set alicat [lindex [lsort [array names ALIAS]] 0]
    set Changed 1
    DisplayAliases
}

proc DisplayAliases {} {
    global ALIAS alicat

    set curYview [.alias.s get]
    .alias.l delete 0 end
    foreach Alias $ALIAS($alicat) {.alias.l insert end $Alias}
    if {[llength $curYview] == 4} {
        .alias.l yview [lindex $curYview 2]
    } elseif {[llength $curYview] == 2} {
        .alias.l yview moveto [lindex $curYview 0]
    }
}

proc InsertAlias {{Title Alias} {AliasIn {}}} {
    global FONT ALIAS ADDRESS alicat
    global Changed Alias Address

    catch {destroy .aa}
    update
    toplevel .aa
    frame .aa.b
    button .aa.b.cancel -text "Cancel" -command {destroy .aa}
    button .aa.b.ok -text "OK" -command \
      {set Alias [.aa.f2.e1 get]; set Address [.aa.f2.e2 get]; destroy .aa}
    frame .aa.f1
    label .aa.f1.l1 -text "Alias:" -anchor e -font $FONT(ProB)
    label .aa.f1.l2 -text "Address:" -anchor e -font $FONT(ProB)
    frame .aa.f2
    entry .aa.f2.e1 -width 40 -font $FONT(ProR)
    entry .aa.f2.e2 -width 40 -font $FONT(ProR)
    pack append .aa.b \
      .aa.b.ok {top expand fill} \
      .aa.b.cancel {top expand fill}
    pack append .aa.f1 \
      .aa.f1.l1 {top fill} \
      .aa.f1.l2 {top fill}
    pack append .aa.f2 \
      .aa.f2.e1 {top fill} \
      .aa.f2.e2 {top fill}
    pack append .aa \
      .aa.f1 {left padx 20 pady 30} \
      .aa.f2 {left padx 20 pady 30} \
      .aa.b {left filly}
     
    wm title .aa $Title
    bind .aa.f2.e1 <space> {break}
    bind .aa.f2.e2 <space> {break}
    bind .aa.f2.e1 <Return> {.aa.b.ok invoke; break} 
    bind .aa.f2.e2 <Return> {.aa.b.ok invoke; break} 
    focus .aa.f2.e1
    grab .aa
    set Alias $AliasIn

    if {$Alias != {}} {
        set Index [lsearch $ALIAS($alicat) $Alias]
        set Address [lindex $ADDRESS($alicat) $Index]
        .aa.f2.e1 delete 0 end
        .aa.f2.e2 delete 0 end
        .aa.f2.e1 insert 0 $Alias
        .aa.f2.e2 insert 0 $Address
        set ALIAS($alicat) [lreplace $ALIAS($alicat) $Index $Index]
        set ADDRESS($alicat) [lreplace $ADDRESS($alicat) $Index $Index]
    } else {
        if {[set Index [lindex [.alias.l curselection] 0]] == ""} {
            set Index [expr [llength $ALIAS($alicat)]]
        }
    }
    
    tkwait window .aa
    if {$Alias == ""} return
    if {$Address == ""} return
    set ALIAS($alicat) [linsert $ALIAS($alicat) $Index $Alias]
    set ADDRESS($alicat) [linsert $ADDRESS($alicat) $Index $Address]
    if [winfo exists .alias] {DisplayAliases}
    set Changed 1
    update
}

proc SaveAlias {} {
    global cmd MH ALIAS ADDRESS alicat
    global Changed 

    if {$MH(AliasFile) == {}} {
        set MH(AliasFile) [Ask "Name of AliasFile:"]
        if {$MH(AliasFile) == {}} {set MH(AliasFile) tkmh.aliases; ConfMH:Write}
    }
    set Fhandle [open [RF $MH(Path)/$MH(AliasFile)] w]

    foreach name [lsort [array names ALIAS]] {
        set numlist [llength $ALIAS($name)]
        for {set i 0} {$i < $numlist} {incr i} {
            puts $Fhandle "$name:[lindex $ALIAS($name) $i]:   [lindex $ADDRESS($name) $i]"
        }
        puts $Fhandle ""
    }

    close $Fhandle
    set Changed 0
}

proc SearchAlias {Alias {resolve 0}} {
    global cmd MH ALIAS ADDRESS

    foreach name [lsort [array names ALIAS]] {
        set result [lsearch $ALIAS($name) $Alias]
        if {$result != -1} {
            set Address [lindex $ADDRESS($name) $result]
            break
        }
    }

    if $resolve {
        switch [string index $Address 0] {
        < -
        + -
        = -
        * {set Address [eval exec $MH(bindir)/ali -nouser -nolist $Alias]}
        default 
           { if {[set i [lsearch $ALIAS($name) $Address]] != -1} {
             set Address [SearchAlias $Address 1]}
           }
        }
    }
    return $Address
}


proc InsertCategory {} {
    global FONT ALIAS ADDRESS alicat Changed

    catch {destroy .aa}
    update
    toplevel .aa
    frame .aa.b
    button .aa.b.cancel -text "Cancel" -command {destroy .aa}
    button .aa.b.ok -text "OK" -command {
        set ALIAS([.aa.f2.e1 get]) {}
        set ADDRESS([.aa.f2.e1 get]) {}
        set alicat [.aa.f2.e1 get]
        destroy .aa
    }
    frame .aa.f1
    label .aa.f1.l1 -text "Category:" -anchor e -font $FONT(ProB)
    frame .aa.f2
    entry .aa.f2.e1 -width 40 -font $FONT(ProR)
    pack append .aa.b \
      .aa.b.ok {top expand fill} \
      .aa.b.cancel {top expand fill}
    pack append .aa.f1 \
      .aa.f1.l1 {top fill} 
    pack append .aa.f2 \
      .aa.f2.e1 {top fill} 
    pack append .aa \
      .aa.f1 {left padx 20 pady 30} \
      .aa.f2 {left padx 20 pady 30} \
      .aa.b {left filly}
     
    wm title .aa "Add Category"
    bind .aa.f2.e1 <space> {break}
    bind .aa.f2.e1 <Return> {.aa.b.ok invoke; break} 
    focus .aa.f2.e1
    grab .aa
    tkwait window .aa
    if [winfo exists .alias] {DisplayAliases}
    set Changed 1
    update
}
